from functools import wraps
from datetime import datetime
from posixpath import join
from pydantic import BaseModel
from typing import Union

from max_ard.session import get_user_session, get_self, ARD_HOSTNAME, ARD_DEV_BRANCH


class UsageArea(BaseModel):
    ''' A model to hold ARD usage by area '''
    fresh_imagery_sqkm: float
    standard_imagery_sqkm: float
    training_imagery_sqkm: float 
    total_imagery_sqkm: float 
    estimate: bool

class UsageCost(BaseModel):
    ''' A model to hold ARD image costs, estimated or consumed
        
        A value of None for a cost means your account does not have pricing set
        for this category. Without set pricing this category can not be ordered.'''

    fresh_imagery_cost: Union[float, None]
    standard_imagery_cost: Union[float, None]
    training_imagery_cost: Union[float, None]
    total_imagery_cost: Union[float, None]
    estimate: bool
    
class UsageLimits(BaseModel):
    ''' A model to hold account limits '''

    limit_sqkm: float
    fresh_imagery_fee_limit: float
    standard_imagery_fee_limit: float
    training_imagery_fee_limit: float 
    annual_subscription_fee_limit: float

class UsageAvailable(BaseModel):
    ''' A model to hold account balances
    
        When returned with a select, these numbers reflect the account balance at the time
        the select was submitted. Usage estimates are not applied to these numbers.'''

    available_sqkm: float
    fresh_imagery_balance: float
    standard_imagery_balance: float
    training_imagery_balance: float
    total_imagery_balance: float



class AdminUsage(BaseModel):
    area: UsageArea
    cost: UsageCost
    limits: UsageLimits
    available: UsageAvailable


    def __str__(self):

        def u(val):
            if val == -1:
                return "unlimited"
            else:
                return f'{val:.2f}'

        return f'''
 Data Usage
 ----------

  Account limit: {u(self.limits.limit_sqkm)} sq.km (${u(self.limits.annual_subscription_fee_limit)})

  Imagery ordered: {self.area.total_imagery_sqkm} sq.km (${self.cost.total_imagery_cost})
  ├ Fresh (< 90 days): {self.area.fresh_imagery_sqkm} sq.km (${self.cost.fresh_imagery_cost})
  ├ Standard (90 days - 3 years): {self.area.standard_imagery_sqkm} sq.km (${self.cost.standard_imagery_cost})
  └ Training (> 3 years): {self.area.training_imagery_sqkm} sq.km (${self.cost.training_imagery_cost})

  Remaining balance: {u(self.available.available_sqkm)} sq.km (${u(self.available.total_imagery_balance)})
  
  '''


class AccountManager():

    def __init__(self, account_id=None, session=None) -> None:
        # we'll be able to pull the account id from the user self endpoint
        # and do this automatically
        if not session:
            session = get_user_session()
        self.session = session
        if not account_id:
            account_id = get_self(self.session)['user']['account_id']
        self.account_id = account_id
        self.base_url = f'https://{ARD_HOSTNAME}/admin{ARD_DEV_BRANCH}/account/{account_id}'
        self.properties = self.get('')['account']

    def api_helper(func):
        @wraps(func)
        def wrapper(self, url, **kwargs):
            url = join(self.base_url, url)
            r = func(self, url, **kwargs)
            # use func.__name__ to enhance error message or generate new exception
            r.raise_for_status()
            if r.status_code == 204:
                return None
            else:
                return r.json()
        return wrapper

    @api_helper
    def get(self, url):
        return self.session.get(url)
    
    @api_helper
    def put(self, url, json=None):
        return self.session.put(url, json=json) 

    @api_helper
    def post(self, url, json=None):
        return self.session.post(url, json=json) 

    @api_helper
    def delete(self, url):
        return self.session.delete(url) 


    ########
    # Usage
    ########

    def _validate_dates(self, *args):
        for date in args:
            if date not in [None, '']:
                try:
                    datetime.strptime(date, '%Y-%m-%d')
                except ValueError:
                    raise ValueError('Dates must be YYYY-MM-DD')
        
    def get_account_usage(self, start_date=None, end_date=None):

        url = f'https://{ARD_HOSTNAME}/usage{ARD_DEV_BRANCH}/account/{self.account_id}'

        self._validate_dates(start_date, end_date)

        params = {
            'start_date': start_date,
            'end_date': end_date
        }

        r = self.session.get(url, params=params)
        r.raise_for_status()
        return AdminUsage(**r.json()['usage'])

    def get_user_usage(self, username=None, start_date=None, end_date=None):
        if not username:
            username = get_self(self.session)['user']['user_id']

        url = f'https://{ARD_HOSTNAME}/usage{ARD_DEV_BRANCH}/user/{username}'

        self._validate_dates(start_date, end_date)

        params = {
            'start_date': start_date,
            'end_date': end_date
        }

        r = self.session.get(url, params=params)
        r.raise_for_status()
        return AdminUsage(**r.json()['usage'])

   